/*
 * This file is part of the ESO UVES Pipeline
 * Copyright (C) 2004,2005 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 */

/*
 * $Author: amodigli $
 * $Date: 2010/09/24 09:32:02 $
 * $Revision: 1.8 $
 * $Name: uves-5_0_0 $
 * $Log: flames_newmatrix.h,v $
 * Revision 1.8  2010/09/24 09:32:02  amodigli
 * put back QFITS dependency to fix problem spot by NRI on FIBER mode (with MIDAS calibs) data
 *
 * Revision 1.6  2009/04/14 07:01:07  amodigli
 * added to CVS (moded from flames tree)
 *
 * Revision 1.8  2007/06/06 08:17:33  amodigli
 * replace tab with 4 spaces
 *
 * Revision 1.7  2007/06/06 07:22:20  jmlarsen
 * Changed return type of nrerror()
 *
 * Revision 1.6  2007/05/07 06:52:17  amodigli
 * fixed compilation warnings
 *
 * Revision 1.5  2007/01/10 11:08:34  jmlarsen
 * Don't define DRS_USE_ORDEF
 *
 * Revision 1.4  2006/10/17 12:33:42  jmlarsen
 * Moved FLAMES source to flames directory
 *
 * Revision 1.4  2006/10/12 11:59:23  jmlarsen
 * Conform to source code template
 *
 * Revision 1.4  2006/08/17 13:56:52  jmlarsen
 * Reduced max line length
 *
 * Revision 1.3  2006/02/28 09:15:22  jmlarsen
 * Minor update
 *
 * Revision 1.2  2005/12/19 16:17:56  jmlarsen
 * Replaced bool -> int
 *
 */
#ifndef FLAMES_NEWMATRIX_H
#define FLAMES_NEWMATRIX_H

#include <flames_uves.h>

void nrerror(const char* error_text);
float *vector(long nl, long nh);
int *ivector(long nl, long nh);
unsigned int *uivector(long nl, long nh);
char *cvector(long nl, long nh);
unsigned char *ucvector(long nl, long nh);
long *lvector(long nl, long nh);
unsigned long int *ulvector(long nl, long nh);
double *dvector(long nl, long nh);
frame_data *fdvector(long nl, long nh);
frame_mask *fmvector(long nl, long nh);
float **matrix(long nrl, long nrh, long ncl, long nch);
char **cmatrix(long nrl, long nrh, long ncl, long nch);
double **dmatrix(long nrl, long nrh, long ncl, long nch);
int **imatrix(long nrl, long nrh, long ncl, long nch);
unsigned long int **ulmatrix(long nrl, long nrh, long ncl, long nch);
long int **lmatrix(long nrl, long nrh, long ncl, long nch);
frame_data **fdmatrix(long nrl, long nrh, long ncl, long nch);
frame_mask **fmmatrix(long nrl, long nrh, long ncl, long nch);
float **submatrix(float **a, long oldrl, long oldrh, long oldcl,
    long newrl, long newcl);
float **convert_matrix(float *a, long nrl, long nrh, long ncl, long nch);
float ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh);
double ***d3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh);
frame_data ***fd3tensor(long nrl, long nrh, long ncl, long nch, long ndl, 
            long ndh);
frame_mask ***fm3tensor(long nrl, long nrh, long ncl, long nch, long ndl, 
            long ndh);
unsigned long int ***ul3tensor(long nrl, long nrh, long ncl, long nch, 
                   long ndl, long ndh);
long int ***l3tensor(long nrl, long nrh, long ncl, long nch, long ndl, 
             long ndh);
long int ****l4tensor(long nal, long nah, long nrl, long nrh, long ncl, 
              long nch, long ndl, long ndh);
void free_vector(float *v, long nl, long nh);
void free_ivector(int *v, long nl, long nh);
void free_uivector(unsigned int *v, long nl, long nh);
void free_cvector(char *v, long nl, long nh);
void free_ucvector(unsigned char *v, long nl, long nh);
void free_lvector(long int *v, long nl, long nh);
void free_ulvector(unsigned long int *v, long nl, long nh);
void free_dvector(double *v, long nl, long nh);
void free_fdvector(frame_data *v, long nl, long nh);
void free_fmvector(frame_mask *v, long nl, long nh);
void free_matrix(float **m, long nrl, long nrh, long ncl, long nch);
void free_cmatrix(char **m, long nrl, long nrh, long ncl, long nch);
void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch);
void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch);
void free_ulmatrix(unsigned long int **m, long nrl, long nrh, long ncl, 
           long nch);
void free_lmatrix(long int **m, long nrl, long nrh, long ncl, long nch);
void free_fdmatrix(frame_data **m, long nrl, long nrh, long ncl, long nch);
void free_fmmatrix(frame_mask **m, long nrl, long nrh, long ncl, long nch);
void free_submatrix(float **b, long nrl, long nrh, long ncl, long nch);
void free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch);
void free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch,
    long ndl, long ndh);
void free_d3tensor(double ***t, long nrl, long nrh, long ncl, long nch,
    long ndl, long ndh);
void free_fd3tensor(frame_data ***t, long nrl, long nrh, long ncl, long nch,
    long ndl, long ndh);
void free_fm3tensor(frame_mask ***t, long nrl, long nrh, long ncl, long nch,
    long ndl, long ndh);
void free_ul3tensor(unsigned long int ***t, long nrl, long nrh, long ncl, 
            long nch,
    long ndl, long ndh);
void free_l3tensor(long int ***t, long nrl, long nrh, long ncl, long nch,
    long ndl, long ndh);
void free_l4tensor(long int ****t, long nal, long nah, long nrl, long nrh, 
           long ncl, long nch, long ndl, long ndh);
void matrix_product(double **, double **, double **, int , int , int );
void matrix_sum(double **, double **, int , int );


#endif
